local log   = require 'loverocks.log'
local etlua = require 'etlua'

local template = {}

local function raw_version(s)
	return string.format("%q", s:gsub("-1$", ""))
end

local warning = [[
--  WARNING: This file is automatically generated and can be rewritten without
--  warning! Make changes to the the loverocks template instead.]]

function template.new_env(vs, name)
	return {
		project_name = name,
		versions = vs,
		raw_version = raw_version,
		loverocks_version = require 'loverocks.version',
		rocks_warning = warning
	}
end

function template.apply(files, env)
	local done = {}
	for name, file in pairs(files) do
		if type(file) == 'table' then
			done[name] = template.apply(file, env)
		elseif not name:match("%.swp$") then

			local d, err = etlua.render(file, env)
			if not d then
				log:error("%s", name .. ": " .. err)
			end

			done[name] = d
		end
	end

	return done
end

return template
